@@ -39,7 +39,10 @@ end = struct
   let ensure b n =
     let len = ref b.len in
     let pos = b.pos in
+    if n > Sys.max_string_length then
+     failwith (Printf.sprintf "Cannot grow string buffer to len %i.\n" n);
     while n > !len do len := !len * 2 done;
+    if !len > Sys.max_string_length then len := Sys.max_string_length;
     let nbuf = String.create !len in
     String.blit b.buf 0 nbuf 0 pos;
     b.buf <- nbuf;
@@ -117,7 +120,7 @@ end
 
 type symbol = {
   mutable sym_pos: int;
-  sym_name: string;
+  mutable sym_name: string;
   mutable value: int32;
   mutable section: [ `Num of int | `Section of section ];
   stype: int;
@@ -135,7 +138,7 @@ and reloc = {
 
 and section = {
   mutable sec_pos: int;
-  sec_name: string;
+  mutable sec_name: string;
   mutable vsize: int32;
   mutable vaddress: int32;
   mutable data:
@@ -145,7 +148,7 @@ and section = {
     | `Sxdata of symbol array
 ];
   mutable relocs: reloc list;
-  sec_opts: int32;
+  mutable sec_opts: int32;
 }
 
 type coff = {
